#pragma once

#include <unordered_map>

#include <QHash>
#include <QList>
#include <QTreeWidgetItem>
#include <QWidget>

#include <clap/clap.h>

QT_BEGIN_NAMESPACE
class QTreeWidget;
class QTreeWidgetItem;
class QLabel;
class QDial;
class QSlider;
QT_END_NAMESPACE

class PluginHost;
class PluginParam;

class PluginParametersWidget : public QWidget {
   Q_OBJECT
public:
   explicit PluginParametersWidget(QWidget *parent, PluginHost &pluginHost);

   class ModuleTreeItem;
   class ParamTreeItem : public QTreeWidgetItem {
   public:
      ParamTreeItem(ModuleTreeItem *parent, PluginParam &param);
      QVariant data(int column, int role) const override;
      void setData(int column, int role, const QVariant &value) override;

      auto &param() { return param_; }
      auto &param() const { return param_; }

   private:
      PluginParam &param_;
   };

   class ModuleTreeItem : public QTreeWidgetItem {
   public:
      ModuleTreeItem(QTreeWidget *parent);
      ModuleTreeItem(ModuleTreeItem *parent, const QString &name);

      void clear();

      ModuleTreeItem &subModule(const QString &name);
      void addItem(ParamTreeItem *item);

      QVariant data(int column, int role) const override;
      void setData(int column, int role, const QVariant &value) override;

   private:
      QString name_;
      QHash<QString, ModuleTreeItem *> modules_;
   };

signals:

private:
   void computeDataModel();
   void paramAdjustedFromPlugin(clap_id paramId);
   void selectionChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous);

   void connectToParam(PluginParam *param);
   void disconnectFromParam();

   void paramInfoChanged();
   void paramValueChanged();
   void sliderValueChanged(int newValue);
   void sliderModulationChanged(int newValue);

   void updateAll();
   void updateParamInfo();
   void updateParamValue();
   void updateParamValueText();
   void updateParamModulation();
   void updateParamIsBeingAjusted();

   static const constexpr int SLIDER_RANGE = 10000;

   PluginHost &_pluginHost;
   QTreeWidget *_treeWidget = nullptr;
   std::unordered_map<clap_id, std::unique_ptr<ParamTreeItem>> _idToParamTreeItem;
   ModuleTreeItem *_rootModuleItem;
   PluginParam *_currentParam = nullptr;

   QLabel *_idLabel = nullptr;
   QLabel *_nameLabel = nullptr;
   QLabel *_moduleLabel = nullptr;
   QLabel *_isAutomatableLabel = nullptr;
   QLabel *_isAutomatablePerNoteIdLabel = nullptr;
   QLabel *_isAutomatablePerKeyLabel = nullptr;
   QLabel *_isAutomatablePerChannelLabel = nullptr;
   QLabel *_isAutomatablePerPortLabel = nullptr;
   QLabel *_isModulatableLabel = nullptr;
   QLabel *_isModulatablePerNoteIdLabel = nullptr;
   QLabel *_isModulatablePerKeyLabel = nullptr;
   QLabel *_isModulatablePerChannelLabel = nullptr;
   QLabel *_isModulatablePerPortLabel = nullptr;
   QLabel *_isPeriodicLabel = nullptr;
   QLabel *_isReadOnlyLabel = nullptr;
   QLabel *_isHiddenLabel = nullptr;
   QLabel *_isBypassLabel = nullptr;
   QLabel *_isSteppedLabel = nullptr;
   QLabel *_minValueLabel = nullptr;
   QLabel *_maxValueLabel = nullptr;
   QLabel *_defaultValueLabel = nullptr;
   QLabel *_isBeingAdjusted = nullptr;
   QLabel *_valueLabel = nullptr;
   QSlider *_valueSlider = nullptr;
   QSlider *_modulationSlider = nullptr;
};
